@@ -77,7 +77,7 @@ add.edge.network<-function(x, tail, head, names.eval=NULL, vals.eval=NULL, edge.
   if(.validLHS(xn,parent.frame())){  #If x not anonymous, set in calling env 
     on.exit(eval.parent(call('<-',xn,x)))
   }
-  x<-.Call("addEdge_R",x,tail,head,names.eval,vals.eval,edge.check, PACKAGE="network")
+  x<-.Call(addEdge_R,x,tail,head,names.eval,vals.eval,edge.check)
   invisible(x)
 }
 
@@ -119,7 +119,7 @@ add.edges.network<-function(x, tail, head, names.eval=NULL, vals.eval=NULL, ...)
     edge.check<-FALSE
   #Pass the inputs to the C side
   xn<-substitute(x)
-  x<-.Call("addEdges_R",x,tail,head,names.eval,vals.eval,edge.check, PACKAGE="network")
+  x<-.Call(addEdges_R,x,tail,head,names.eval,vals.eval,edge.check)
   if(.validLHS(xn,parent.frame())){  #If x not anonymous, set in calling env 
     on.exit(eval.parent(call('<-',xn,x)))
   }
@@ -159,7 +159,7 @@ add.vertices.network<-function(x, nv, vattr=NULL, last.mode=TRUE, ...){
       on.exit(eval.parent(call('<-',xn,x)))
     }
     if(last.mode||(!is.bipartite(x))){
-      x<-.Call("addVertices_R",x,nv,vattr, PACKAGE="network")
+      x<-.Call(addVertices_R,x,nv,vattr)
     }else{
       
       nr<-nv
@@ -167,7 +167,7 @@ add.vertices.network<-function(x, nv, vattr=NULL, last.mode=TRUE, ...){
       nnew<-nr+nc
       nold<-network.size(x)
       bip<-x%n%"bipartite"
-      x<-.Call("addVertices_R", x, nv, vattr, PACKAGE = "network")
+      x<-.Call(addVertices_R, x, nv, vattr)
       
       if(nr>0){
         if(bip>0)
@@ -199,7 +199,7 @@ delete.edge.attribute<-function(x,attrname){
     stop("delete.edge.attribute requires an argument of class network.")
   #Remove the edges
   xn<-substitute(x)
-  x<-.Call("deleteEdgeAttribute_R",x,attrname, PACKAGE="network")
+  x<-.Call(deleteEdgeAttribute_R,x,attrname)
   if(.validLHS(xn,parent.frame())){  #If x not anonymous, set in calling env 
     on.exit(eval.parent(call('<-',xn,x)))
   }
@@ -219,7 +219,7 @@ delete.edges<-function(x,eid){
     if((min(eid)<1)|(max(eid)>length(x$mel)))
       stop("Illegal edge in delete.edges.\n")
     #Remove the edges
-    x<-.Call("deleteEdges_R",x,eid, PACKAGE="network")
+    x<-.Call(deleteEdges_R,x,eid)
     if(.validLHS(xn,parent.frame())){  #If x not anonymous, set in calling env 
       on.exit(eval.parent(call('<-',xn,x)))
     }
@@ -235,7 +235,7 @@ delete.network.attribute<-function(x,attrname){
     stop("delete.network.attribute requires an argument of class network.")
   #Remove the edges
   xn<-substitute(x)
-  x<-.Call("deleteNetworkAttribute_R",x,attrname, PACKAGE="network")
+  x<-.Call(deleteNetworkAttribute_R,x,attrname)
   if(.validLHS(xn,parent.frame())){  #If x not anonymous, set in calling env 
     on.exit(eval.parent(call('<-',xn,x)))
   }
@@ -252,7 +252,7 @@ delete.vertex.attribute<-function(x,attrname){
   #Remove the attribute (or do nothing, if there are no vertices)
   if(network.size(x)>0){
     xn<-substitute(x)
-    x<-.Call("deleteVertexAttribute_R",x,attrname, PACKAGE="network")
+    x<-.Call(deleteVertexAttribute_R,x,attrname)
     if(.validLHS(xn,parent.frame())){  #If x not anonymous, set in calling env 
       on.exit(eval.parent(call('<-',xn,x)))
     }
@@ -276,7 +276,7 @@ delete.vertices<-function(x,vid){
       m1v<-get.network.attribute(x,"bipartite")  #How many mode 1 verts?
       set.network.attribute(x,"bipartite",m1v-sum(vid<=m1v))
     }
-    x<-.Call("deleteVertices_R",x,vid, PACKAGE="network")
+    x<-.Call(deleteVertices_R,x,vid)
     if(.validLHS(xn,parent.frame())){  #If x not anonymous, set in calling env 
       on.exit(eval.parent(call('<-',xn,x)))
     }
@@ -286,27 +286,39 @@ delete.vertices<-function(x,vid){
 
 
 # Retrieve a specified edge attribute from edge list el.  The attribute
-# is returned as a list, regardless of type.
+# is returned as a list, unless unlist is TRUE. 
+# if deleted.edges.omit is TRUE, then only attribute values on existing (non-null) edges will be returned.
+# if na.omit is TRUE, than values corresponding to 'missing' edges (edges with attribute 'na' set to TRUE) should be ommited. (NULL edgs count as not-missing)
+# If null.na is TRUE, then values corresponding to  edges for which the attribute name was never set will be set to NA.  Otherwise, they will be NULL, which means they will be included when unlist=TRUE 
 #
-get.edge.attribute<-function(el, attrname, unlist=TRUE){
-  if (is.network(el)){
-    # call get.edge.value instead
-    return(get.edge.value(el,attrname=attrname,unlist=unlist))
-  } 
-  x <- lapply(lapply(el,"[[","atl"),"[[",attrname)
-  if(unlist){unlist(x)}else{x}
+get.edge.attribute<-function(el, attrname, unlist=TRUE,na.omit=FALSE,null.na=FALSE,deleted.edges.omit=FALSE){
+  if (is.network(el)) el <- el$mel
+
+  if (!is.list(el))
+    stop("el must be a network object or a list.")
+
+  if (!is.character(attrname))
+    stop("attrname must be a character vector.")
+
+  if (!is.logical(unlist) || !is.logical(na.omit) || !is.logical(null.na) ||
+      !is.logical(deleted.edges.omit))
+    stop("na.omit, null.na, deleted.edges.omit must be a logical vector.")
+
+  edges <- .Call(getEdgeAttribute_R,el,attrname,na.omit,null.na,deleted.edges.omit)
+
+  if(unlist)
+    unlist(edges)
+  else
+    edges
 }
 
 
-# Retrieve a specified edge attribute from all edges in x.  The attribute
-# is returned as a list, regardless of type.
+# Retrieve a specified edge attribute from all edges in x.
 #
-get.edge.value<-function(x, attrname, unlist=TRUE){
-  y <- lapply(lapply(x$mel,"[[","atl"),"[[",attrname)
-  if(unlist){unlist(y)}else{y}
+get.edge.value<-function(x, attrname, unlist=TRUE, na.omit=FALSE, null.na=FALSE, deleted.edges.omit=FALSE){
+  get.edge.attribute(x,attrname,unlist,na.omit,null.na,deleted.edges.omit)
 }
 
-
 # Retrieve the ID numbers for all edges incident on v, in network x.  
 # Outgoing or incoming edges are specified by neighborhood, while na.omit 
 # indicates whether or not missing edges should be omitted.  The return value
@@ -328,7 +340,7 @@ get.edgeIDs<-function(x, v, alter=NULL, neighborhood=c("out","in","combined"), n
   else
     neighborhood=match.arg(neighborhood)
   #Do the deed
-  .Call("getEdgeIDs_R",x,v,alter,neighborhood,na.omit, PACKAGE="network")
+  .Call(getEdgeIDs_R,x,v,alter,neighborhood,na.omit)
 }
 
 
@@ -353,7 +365,7 @@ get.edges<-function(x, v, alter=NULL, neighborhood=c("out","in","combined"), na.
   else
     neighborhood=match.arg(neighborhood)
   #Do the deed
-  .Call("getEdges_R",x,v,alter,neighborhood,na.omit, PACKAGE="network")
+  .Call(getEdges_R,x,v,alter,neighborhood,na.omit)
 }
 
 
@@ -362,48 +374,83 @@ get.edges<-function(x, v, alter=NULL, neighborhood=c("out","in","combined"), na.
 # return the edge cut (along with the associated vertices and meta-data) as
 # a bipartite network.
 #
-get.inducedSubgraph<-function(x, v, alters=NULL){
+get.inducedSubgraph<-function(x, v, alters=NULL, eid=NULL){
   #Check to be sure we were called with a network
   if(!is.network(x))
     stop("get.inducedSubgraph requires an argument of class network.")
   #Do some reality checking
   n<-network.size(x)
-  if((length(v)<1)||any(is.na(v))||any(v<1)||any(v>n))
-    stop("Illegal vertex selection in get.inducedSubgraph")
-  if(!is.null(alters)){
-    if((length(alters)<1)||any(is.na(alters))||any(alters<1)||any(alters>n)|| any(alters%in%v))
-      stop("Illegal vertex selection (alters) in get.inducedSubgraph")
+  
+  # are we doing this via eids, or v and alters
+  if (is.null(eid)){  # do checks for v and alters
+    if((length(v)<1)||any(is.na(v))||any(v<1)||any(v>n))
+      stop("Illegal vertex selection in get.inducedSubgraph")
+    if(!is.null(alters)){
+      if((length(alters)<1)||any(is.na(alters))||any(alters<1)||any(alters>n)|| any(alters%in%v))
+        stop("Illegal vertex selection (alters) in get.inducedSubgraph")
+    }
+    if (!is.null(eid)){
+      warning('eid argument to get.inducedSubgraph ignored when using v or alter argument')
+    }
+  } else { # do checks for eids
+    if (!is.numeric(eid)){
+      stop('eid must be a numeric vector of edge ids')
+    }
+    if (!missing(v)){
+      warning('v argument to get.inducedSubgraph ignored when using eid argument')
+    }
+    if (!is.null(alters)){
+      warning('alters argument to get.inducedSubgraph ignored when using eid argument')
+    }
+    # check that eids are valid
+    if (any(!eid%in%valid.eids(x))){
+      stop('eid argument contains non-valid edge ids')
+    }
+    
   }
+  
   #Start by making a copy of our target network (yes, this can be wasteful)
-  new<-network.copy(x)
-  #Now, strip out what is needed, and/or permute in the two-mode case
-  if(is.null(alters)){                    #Simple case
-    delete.vertices(new,(1:n)[-v])           #Get rid of everyone else
-  }else{                                  #Really an edge cut, but w/vertices
-    nv<-length(v)
-    na<-length(alters)
-    newids<-sort(c(v,alters))
-    newv<-match(v,newids)
-    newalt<-match(alters,newids)
-    delete.vertices(new,(1:n)[-c(v,alters)])  #Get rid of everyone else
-    permute.vertexIDs(new,c(newv,newalt))    #Put the new vertices first
-    #Remove within-group edges
-    for(i in 1:nv)
-      for(j in (i:nv)[-1]){
-        torem<-get.edgeIDs(new,i,alter=j,neighborhood="combined",na.omit=FALSE)
-        if(length(torem)>0)
-          delete.edges(new,torem)
-      }
-    for(i in (nv+1):(nv+na))
-      for(j in (i:(nv+na))[-1]){
-        torem<-get.edgeIDs(new,i,alter=j,neighborhood="combined",na.omit=FALSE)
-        if(length(torem)>0)
-          delete.edges(new,torem)
-      }
-    new%n%"bipartite"<-nv   #Set bipartite attribute
+  #TODO: in most cases, probably faster to create a new network and only copy over what is needed
+  newNet<-network.copy(x)
+  
+  if (is.null(eid)){  # using v and alter
+    #Now, strip out what is needed, and/or permute in the two-mode case
+    if(is.null(alters)){                    #Simple case
+      delete.vertices(newNet,(1:n)[-v])           #Get rid of everyone else
+    }else{                                  #Really an edge cut, but w/vertices
+      nv<-length(v)
+      na<-length(alters)
+      newids<-sort(c(v,alters))
+      newv<-match(v,newids)
+      newalt<-match(alters,newids)
+      delete.vertices(newNet,(1:n)[-c(v,alters)])  #Get rid of everyone else
+      permute.vertexIDs(newNet,c(newv,newalt))    #Put the new vertices first
+      #Remove within-group edges
+      for(i in 1:nv)
+        for(j in (i:nv)[-1]){
+          torem<-get.edgeIDs(newNet,i,alter=j,neighborhood="combined",na.omit=FALSE)
+          if(length(torem)>0)
+            delete.edges(newNet,torem)
+        }
+      for(i in (nv+1):(nv+na))
+        for(j in (i:(nv+na))[-1]){
+          torem<-get.edgeIDs(newNet,i,alter=j,neighborhood="combined",na.omit=FALSE)
+          if(length(torem)>0)
+            delete.edges(newNet,torem)
+        }
+      newNet%n%"bipartite"<-nv   #Set bipartite attribute
+    }
+  } else {  # using eids instead of v and alters
+    # delete all the edges not in eid
+    removeEid<-setdiff(valid.eids(newNet),eid)
+    delete.edges(newNet,removeEid)
+    # find the set of vertices incident on the remaining edges
+    v<-unique(c(unlist(sapply(newNet$mel, "[[", "outl")),unlist(sapply(newNet$mel, "[[", "inl"))))
+    removeV<-setdiff(seq_len(network.size(newNet)),v)
+    delete.vertices(newNet,removeV)
   }
   #Return the updated object
-  new
+  newNet
 }
 
 
@@ -434,7 +481,7 @@ get.neighborhood<-function(x, v, type=c("out","in","combined"), na.omit=TRUE){
   else
     type=match.arg(type)
   #Do the deed
-  .Call("getNeighborhood_R",x,v,type,na.omit, PACKAGE="network")
+  .Call(getNeighborhood_R,x,v,type,na.omit)
 }
 
 
@@ -494,7 +541,7 @@ is.adjacent<-function(x,vi,vj,na.omit=FALSE){
     vj<-rep(vj,length=max(length(vi),length(vj)))
   }
   #Do the deed
- .Call("isAdjacent_R",x,vi,vj,na.omit, PACKAGE="network")
+ .Call(isAdjacent_R,x,vi,vj,na.omit)
 }
 
 
@@ -551,7 +598,7 @@ is.na.network<-function(x){
   #Create an empty network with the same properties as x
   y<-network.initialize(network.size(x),directed=is.directed(x), hyper=is.hyper(x),loops=has.loops(x),multiple=is.multiplex(x), bipartite=x%n%"bipartite")
   #Add the missing edges of x to y
-  y<-.Call("isNANetwork_R",x,y,PACKAGE="network")
+  y<-.Call(isNANetwork_R,x,y)
   #Return the updated network 
   y
 }
@@ -651,7 +698,7 @@ network.edgecount<-function(x,na.omit=TRUE){
   if(!is.network(x))
     stop("network.edgecount requires an argument of class network.\n")
   #Return the edge count
-  .Call("networkEdgecount_R",x,na.omit, PACKAGE="network")
+  .Call(networkEdgecount_R,x,na.omit)
 }
 
 
@@ -717,7 +764,7 @@ permute.vertexIDs<-function(x,vids){
   }
   #Return the permuted graph
   xn<-substitute(x)
-  x<-.Call("permuteVertexIDs_R",x,vids, PACKAGE="network")
+  x<-.Call(permuteVertexIDs_R,x,vids)
   if(.validLHS(xn,parent.frame())){  #If x not anonymous, set in calling env 
     on.exit(eval.parent(call('<-',xn,x)))
   }
@@ -780,7 +827,7 @@ set.edge.attribute<-function(x,attrname,value,e=seq_along(x$mel)){
         }
       }
       #Do the deed, call the set single value version
-      x<-.Call("setEdgeAttribute_R",x,attrname,value,e, PACKAGE="network")
+      x<-.Call(setEdgeAttribute_R,x,attrname,value,e)
     } else { # we will be setting multiple values
       if (length(attrname)!=length(value)){
         stop("the 'value' attribute must have an element corresponding to each attribute name in 'attrname' in set.edge.attribute")
@@ -811,7 +858,7 @@ set.edge.attribute<-function(x,attrname,value,e=seq_along(x$mel)){
         
       }
       #Do the deed, call the set multiple version
-      x<-.Call("setEdgeAttributes_R",x,attrname,value,e, PACKAGE="network")
+      x<-.Call(setEdgeAttributes_R,x,attrname,value,e)
     }
     if(.validLHS(xn,parent.frame())){  #If x not anonymous, set in calling env 
       on.exit(eval.parent(call('<-',xn,x)))
@@ -846,7 +893,7 @@ set.edge.value<-function(x,attrname,value,e=seq_along(x$mel)){
   }
   #Do the deed
   xn<-substitute(x)
-  x<-.Call("setEdgeValue_R",x,attrname,value,e, PACKAGE="network")
+  x<-.Call(setEdgeValue_R,x,attrname,value,e)
   if(.validLHS(xn,parent.frame())){  #If x not anonymous, set in calling env 
     on.exit(eval.parent(call('<-',xn,x)))
   }
@@ -873,7 +920,7 @@ set.network.attribute<-function(x,attrname,value){
   }
   #Do the deed
   xn<-substitute(x)
-  x<-.Call("setNetworkAttribute_R",x,attrname,value,PACKAGE="network")
+  x<-.Call(setNetworkAttribute_R,x,attrname,value)
   if(.validLHS(xn,parent.frame())){  #If x not anonymous, set in calling env 
     on.exit(eval.parent(call('<-',xn,x)))
   }
@@ -942,7 +989,7 @@ set.vertex.attribute<-function(x,attrname,value,v=seq_len(network.size(x))){
       }
     }
     # call older singular value version
-    x<-.Call("setVertexAttribute_R",x,attrname,value,v, PACKAGE="network")
+    x<-.Call(setVertexAttribute_R,x,attrname,value,v)
   } else { # setting multiple values
     if (length(value)!=length(attrname)){
       stop("the 'value' attribute must have an element corresponding to each attribute name in 'attrnames' in set.vertex.attribute")
@@ -972,7 +1019,7 @@ set.vertex.attribute<-function(x,attrname,value,v=seq_len(network.size(x))){
       })
     }
     # call multiple value version
-    x<-.Call("setVertexAttributes_R",x,attrname,value,v, PACKAGE="network")
+    x<-.Call(setVertexAttributes_R,x,attrname,value,v)
   } # end setting multiple values
   #Do the deed
   
